<?php

namespace Drupal\webform_advancedqueue\Plugin\WebformHandler;

use Drupal\advancedqueue\Job;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Sends webform submissions to an Advanced Queue job.
 *
 * @WebformHandler(
 *   id = "webform_advancedqueue",
 *   label = @Translation("Webform Advanced Queue"),
 *   category = @Translation("Queue"),
 *   description = @Translation("Sends webform submissions to an Advanced Queue job"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 * )
 */
class AdvancedQueueWebformHandler extends WebformHandlerBase {
  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The job handler manager.
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $jobHandlerManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->jobHandlerManager = $container->get('plugin.manager.advancedqueue_job_type');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'queue_id' => '',
      'job_type' => '',
      'debug' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
    ];

    // List available queues.
    $queues = $this->entityTypeManager->getStorage('advancedqueue_queue')->loadMultiple();
    $queue_options = [];
    foreach ($queues as $queue) {
      $queue_options[$queue->id()] = $queue->label();
    }
    $form['settings']['queue_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Queue'),
      '#description' => $this->t('Select the Advanced Queue to use.'),
      '#options' => $queue_options,
      '#default_value' => $this->configuration['queue_id'],
      '#required' => TRUE,
    ];

    if (empty($queue_options)) {
      $form['settings']['queue_id']['#description'] .= ' ' . $this->t('No Advanced Queues are defined. Please create one first.');
      $form['#disabled'] = TRUE;
    }

    // List available job types.
    $job_handlers = $this->jobHandlerManager->getDefinitions();
    $job_type_options = [];
    foreach ($job_handlers as $job_type => $definition) {
      $job_type_options[$job_type] = $definition['label'] ?? $job_type;
    }
    $form['settings']['job_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Job type'),
      '#description' => $this->t('Select the Advanced Queue job type to use.'),
      '#options' => $job_type_options,
      '#default_value' => $this->configuration['job_type'],
      '#required' => TRUE,
    ];

    if (empty($job_type_options)) {
      $form['settings']['job_type']['#description'] .= ' ' . $this->t('No job types are available.');
      $form['#disabled'] = TRUE;
    }

    $form['settings']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debugging'),
      '#description' => $this->t('If checked, every handler method invoked will be displayed onscreen to all users.'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['debug'],
    ];

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['queue_id'] = $form_state->getValue('queue_id');
    $this->configuration['job_type'] = $form_state->getValue('job_type');
    $this->configuration['debug'] = (bool) $form_state->getValue('debug');
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $configuration = $this->getConfiguration();
    $settings = $configuration['settings'];
    $settings['queue_id'] = $this->configuration['queue_id'];
    $settings['job_type'] = $this->configuration['job_type'];
    return [
      '#settings' => $settings,
    ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $state = $webform_submission->getWebform()->getSetting('results_disabled') ? WebformSubmissionInterface::STATE_COMPLETED : $webform_submission->getState();
    if ($state == WebformSubmissionInterface::STATE_COMPLETED) {
      $data = $webform_submission->getData();
      $payload = [
        'data' => $data,
        'webform_id' => $webform_submission->getWebform()->id(),
      ];

      unset($data['in_draft']);
      try {
        /** @var \Drupal\advancedqueue\Entity\Queue|null $queue */
        $queue = $this->entityTypeManager->getStorage('advancedqueue_queue')->load($this->configuration['queue_id']);
        if ($queue) {
          $job = Job::create($this->configuration['job_type'], $payload);
          $queue->enqueueJob($job);
          $variables = [
            '@queue_id' => $this->configuration['queue_id'],
            '@job_type' => $this->configuration['job_type'],
            '@data' => json_encode($data),
          ];
          $this->getLogger('webform_advancedqueue')->notice('Submission sent to Advanced Queue (@queue_id) job type @job_type', $variables);
          if ($this->configuration['debug']) {
            $this->messenger()->addWarning($this->t('Following data has been sent to Advanced Queue (@queue_id) job type @job_type: @data', $variables), TRUE);
          }
        }
        else {
          $this->getLogger('webform_advancedqueue')->error('Selected queue not found.');
        }
      }
      catch (\Exception $e) {
        $this->getLogger('webform_advancedqueue')->error($e->getMessage());
      }
    }
  }

}
